@@ -29,13 +29,12 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
   m_pLabel_Mode(0),
   m_pLabel_userlevel(0),
   m_pBox_Mode(0),
-  m_pBox_Databases(0),
   m_pBox_Tables(0),
-  m_pDialog_Databases(0),
   m_pDialog_Tables(0),
   m_pDialog_Fields(0),
-  m_pDialog_Relationships(0)
-
+  m_pDialog_Relationships(0),
+  m_pDialogConnection(0),
+  m_pDialogConnectionFailed(0)
 {
   //Load widgets from glade file:
   refGlade->get_widget("label_table_name", m_pLabel_Table);
@@ -46,18 +45,19 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
   //m_pLabel_Mode->set_text(gettext("No database selected.\n Use the Navigation menu, or open a previous Glom document."));
   
   //Load the Glade file and instantiate its widgets to get the dialog stuff:
+
   try
   {
-    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "box_navigation_connection");
+    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_connection");
 
-    refXml->get_widget_derived("box_navigation_connection", m_pBox_Databases);
-    m_pDialog_Databases = new Dialog_Glom(m_pBox_Databases);
+    refXml->get_widget_derived("dialog_connection", m_pDialogConnection);
   }
   catch(const Gnome::Glade::XmlError& ex)
   {
     std::cerr << ex.what() << std::endl;
   }
 
+  
   try
   {
     Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "box_navigation_tables");
@@ -95,15 +95,23 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
     std::cerr << ex.what() << std::endl;
   }
 
+  try
+  {
+    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_error_connection");
+
+    refXml->get_widget("dialog_error_connection", m_pDialogConnectionFailed);
+  }
+  catch(const Gnome::Glade::XmlError& ex)
+  {
+    std::cerr << ex.what() << std::endl;
+  }
+  
+  
+
   m_Mode = MODE_None;
   m_Mode_Previous = MODE_None;
 
   Gtk::Window* pWindow = get_app_window();
-  if(pWindow)
-    m_pDialog_Databases->set_transient_for(*pWindow);
-
-  m_pDialog_Databases->get_vbox()->pack_start(*m_pBox_Databases);
-  m_pBox_Databases->show_all();
 
   if(pWindow)
     m_pDialog_Tables->set_transient_for(*pWindow);
@@ -113,7 +121,6 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
   m_pBox_Tables->show_all();
 
   //Connect signals:
-  m_pBox_Databases->signal_selected.connect(sigc::mem_fun(*this, &Frame_Glom::on_box_databases_selected));
   m_pBox_Tables->signal_selected.connect(sigc::mem_fun(*this, &Frame_Glom::on_box_tables_selected));
 
   
@@ -121,10 +128,10 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
 
   //Fill Composite View:
   //This means that set_document and load/save are delegated to these children:
-  add_view(m_pBox_Databases);
   add_view(m_pBox_Tables);
   add_view(m_pDialog_Fields); //Also a composite view.
   add_view(m_pDialog_Relationships); //Also a composite view.
+  add_view(m_pDialogConnection); //Also a composite view.
   add_view(&m_Notebook_Data); //Also a composite view.
   add_view(&m_Notebook_Find); //Also a composite view.
   
@@ -134,30 +141,45 @@ Frame_Glom::Frame_Glom(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
 
 Frame_Glom::~Frame_Glom()
 {
-  if(m_pDialog_Databases)
-  {
-      delete m_pDialog_Databases;
-      m_pDialog_Databases = 0;
-  }
-
   if(m_pDialog_Tables)
   {
-      delete m_pDialog_Tables;
-      m_pDialog_Tables = 0;
+    delete m_pDialog_Tables;
+    m_pDialog_Tables = 0;
   }
+
+  if(m_pDialogConnection)
+  {
+    delete m_pDialogConnection;
+    m_pDialogConnection = 0;
+  }
+
+  if(m_pDialogConnectionFailed)
+  {
+    delete m_pDialogConnectionFailed;
+    m_pDialogConnectionFailed = 0;
+  }
+
+  if(m_pDialog_Relationships)
+  {
+    delete m_pDialog_Relationships;
+    m_pDialog_Relationships = 0;
+  }
+
+  if(m_pDialog_Relationships)
+  {
+    delete m_pDialog_Relationships;
+    m_pDialog_Relationships = 0;
+  }
+
+  
 }
 
-void Frame_Glom::on_box_databases_selected(Glib::ustring strName)
+void Frame_Glom::set_databases_selected(const Glib::ustring& strName)
 {
-  m_pDialog_Databases->hide(); //cause_close();
+  //m_pDialog_Databases->hide(); //cause_close();
 
   get_document()->set_connection_database(strName);
 
-  //Enable more menus:
-  App_Glom* pApp = dynamic_cast<App_Glom*>(get_app_window());
-  if(pApp)
-    pApp->on_database_selected(true);
-
   do_menu_Navigate_Table(true /* open default */);
 }
 
@@ -215,7 +237,12 @@ bool Frame_Glom::set_mode(enumModes mode)
 void Frame_Glom::alert_no_table()
 {
   //Ask user to choose a table first:
-  show_ok_dialog(gettext("No database chosen"), gettext("You must choose a database table first.\n Use the Navigation menu, or load a previous document."));
+  Gtk::Window* pWindowApp = get_app_window();
+  if(pWindowApp)
+  {
+    //TODO: Obviously this document should have been deleted when the database-creation was cancelled.
+    show_ok_dialog(gettext("No database"), gettext("This document does not specify any database. Maybe the document creation was cancelled before the database could be created."), *pWindowApp);
+  }
 }
 
 void Frame_Glom::show_table(const Glib::ustring& strTableName)
@@ -242,20 +269,20 @@ void Frame_Glom::show_table(const Glib::ustring& strTableName)
     App_Glom* pApp = dynamic_cast<App_Glom*>(get_app_window());
     if(pApp)
       on_userlevel_changed(pApp->get_userlevel());
-    
+ 
     switch(m_Mode)
     {
       case(MODE_Data):
       {
         strMode = gettext("Data");
-        m_Notebook_Data.init_db_details( m_pBox_Databases->get_database_name(), m_strTableName);
+        m_Notebook_Data.init_db_details(m_strTableName);
         set_mode_widget(m_Notebook_Data);
         break;
       }
       case(MODE_Find):
       {
         strMode = gettext("Find");
-        m_Notebook_Find.init_db_details( m_pBox_Databases->get_database_name(), m_strTableName);
+        m_Notebook_Find.init_db_details(m_strTableName);
         set_mode_widget(m_Notebook_Find);
         break;
       }
@@ -275,7 +302,7 @@ void Frame_Glom::show_table(const Glib::ustring& strTableName)
   show_all();
 }
 
-void Frame_Glom::on_menu_userlevel_Developer(const Glib::RefPtr<Gtk::RadioAction>& action)
+void Frame_Glom::on_menu_userlevel_Developer(const Glib::RefPtr<Gtk::RadioAction>& action, const Glib::RefPtr<Gtk::RadioAction>& operator_action)
 {
   if(action && action->get_active())
   {
@@ -284,7 +311,19 @@ void Frame_Glom::on_menu_userlevel_Developer(const Glib::RefPtr<Gtk::RadioAction
     {
       //Avoid double signals:
       //if(document->get_userlevel() != AppState::USERLEVEL_DEVELOPER)
-        document->set_userlevel(AppState::USERLEVEL_DEVELOPER);
+      bool test = document->set_userlevel(AppState::USERLEVEL_DEVELOPER);
+        
+      //If this was not possible then revert the menu:
+      if(!test)
+      {
+        Gtk::MessageDialog dialog(gettext("Developer mode is not available. Check that you have sufficient database access rights and that the glom file is not read-only."), Gtk::MESSAGE_WARNING);
+        dialog.run();
+        
+        //This causes an endless loop, but it is not recursive so we can't block it.
+        //TODO: Submit GTK+ bug.
+        //action->set_active(false);
+        operator_action->set_active();
+      }
     }
   }
 }
@@ -315,12 +354,13 @@ void Frame_Glom::on_menu_Mode_Find()
     show_table(m_strTableName);
 }
 
+/*
 void Frame_Glom::on_menu_Navigate_Database()
 {
   do_menu_Navigate_Database();
 }
 
-void Frame_Glom::do_menu_Navigate_Database(bool bUseList /* = true */)
+void Frame_Glom::do_menu_Navigate_Database(bool bUseList)
 {
   m_pBox_Databases->set_use_list(bUseList);
   m_pBox_Databases->load_from_document();
@@ -338,6 +378,7 @@ void Frame_Glom::do_menu_Navigate_Database(bool bUseList /* = true */)
   //m_frame.add(*m_pBox_Databases);
   //m_frame.show_all();
 }
+*/
 
 void Frame_Glom::on_menu_Navigate_Table()
 {
@@ -348,11 +389,11 @@ void Frame_Glom::do_menu_Navigate_Table(bool open_default)
 {
   if(get_document()->get_connection_database().empty())
   {
-    show_ok_dialog(gettext("No database chosen"), gettext("You must choose a database first.\n Use the Navigation|Database menu item, or load a previous document."));
+    alert_no_table();
   }
   else
   {
-    m_pBox_Tables->init_db_details( get_document()->get_connection_database());
+    m_pBox_Tables->init_db_details();
 
     Glib::ustring default_table_name;
     if(open_default)
@@ -369,6 +410,7 @@ void Frame_Glom::do_menu_Navigate_Table(bool open_default)
     {
       //Let the user choose a table:
       //m_pDialog_Tables->set_policy(false, true, false); //TODO_port
+      //m_pDialog_Tables->load_from_document(); //Refresh
       m_pDialog_Tables->show();
     }
   }
@@ -403,7 +445,7 @@ Gtk::Window* Frame_Glom::get_app_window()
 
 }
 
-void Frame_Glom::show_ok_dialog(const Glib::ustring& title, const Glib::ustring& message)
+void Frame_Glom::show_ok_dialog(const Glib::ustring& title, const Glib::ustring& message, Gtk::Window& parent)
 {
   Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_information");
   Gtk::Dialog* dialog = 0;
@@ -416,9 +458,7 @@ void Frame_Glom::show_ok_dialog(const Glib::ustring& title, const Glib::ustring&
   label->set_text(text);
   label->set_use_markup();
 
-  Gtk::Window* pWindowApp = get_app_window();
-  if(pWindowApp)
-   dialog->set_transient_for(*pWindowApp);
+  dialog->set_transient_for(parent);
 
   dialog->run();
   delete dialog;
@@ -427,7 +467,8 @@ void Frame_Glom::show_ok_dialog(const Glib::ustring& title, const Glib::ustring&
 void Frame_Glom::on_Notebook_Find(Glib::ustring strWhereClause)
 {
   on_menu_Mode_Data();
-  m_Notebook_Data.init_db_details( m_pBox_Databases->get_database_name(), m_strTableName, strWhereClause);
+
+  m_Notebook_Data.init_db_details(m_strTableName, strWhereClause);
   m_Notebook_Data.select_page_for_find_results();
 }
 
@@ -479,7 +520,7 @@ void Frame_Glom::update_table_in_document_from_database()
 
   //Get the fields information from the database:
   Base_DB::type_vecFields fieldsDatabase = Base_DB::get_fields_for_table_from_database(m_strTableName);
-
+  
   Document_Glom* pDoc = dynamic_cast<const Document_Glom*>(get_document());
   if(pDoc)
   {
@@ -533,8 +574,9 @@ void Frame_Glom::update_table_in_document_from_database()
         fieldsActual.push_back(field);
       }
       else
+      {
         document_must_to_be_updated = true; //Something changed.
-      
+      }
     }
           
     if(document_must_to_be_updated)
@@ -576,7 +618,7 @@ void Frame_Glom::on_menu_developer_fields()
   else
   {
     m_pDialog_Fields->set_transient_for(*get_app_window());
-    m_pDialog_Fields->init_db_details( m_pBox_Databases->get_database_name(), m_strTableName);
+    m_pDialog_Fields->init_db_details(m_strTableName);
     m_pDialog_Fields->show();
   }
 }
@@ -591,7 +633,7 @@ void Frame_Glom::on_menu_developer_relationships()
   else
   {
     m_pDialog_Relationships->set_transient_for(*get_app_window());
-    m_pDialog_Relationships->init_db_details( m_pBox_Databases->get_database_name(), m_strTableName);
+    m_pDialog_Relationships->init_db_details(m_strTableName);
     m_pDialog_Relationships->show();
   }
 }
@@ -625,6 +667,135 @@ void Frame_Glom::on_developer_dialog_hide()
 void Frame_Glom::on_menu_developer_recreate_structure()
 {
 }
+
+bool Frame_Glom::connection_request_password_and_attempt()
+{
+  while(true) //Loop until a return
+  {
+    //Ask for connection details:
+    m_pDialogConnection->load_from_document(); //Get good defaults.
+    int response = m_pDialogConnection->run();
+    m_pDialogConnection->hide();
+
+    if(response == Gtk::RESPONSE_OK)
+    {
+      try
+      {
+        //TODO: Remove any previous database setting?
+        sharedptr<SharedConnection> sharedconnection = m_pDialogConnection->connect_to_server_with_connection_settings();
+        return true; //Succeeeded, because no exception was thrown.
+      }
+      catch(const ExceptionConnection& ex)
+      {
+        g_warning("Frame_Glom::connection_request_password_and_attempt(): caught exception.");
+         
+        if(ex.get_failure_type() == ExceptionConnection::FAILURE_NO_SERVER)
+        {
+          //Warn the user, and let him try again:
+          int response = m_pDialogConnectionFailed->run();
+          m_pDialogConnectionFailed->hide();
+
+          if(response != Gtk::RESPONSE_OK)
+            return false; //The user cancelled.
+        }
+        else
+        {
+          g_warning("Frame_Glom::connection_request_password_and_attempt(): rethrowing exception.");
+
+          //The connection to the server is OK, but the specified database does not exist:
+          throw ex; //Pass it on for the caller to handle.
+          return false;
+        }
+      }
+      
+      //Try again.
+    }
+    else
+      return false; //The user cancelled.
+  }
+}
+
+bool Frame_Glom::create_database(const Glib::ustring& database_name, bool request_password)
+{
+  //Ask for connection details:
+  bool connection_possible = false;
+  try
+  {
+    if(request_password)
+      connection_possible = connection_request_password_and_attempt(); //If it succeeded and the user did not cancel.
+    else
+      connection_possible = true; //Assume that connection details are already coorect.
+  }
+  catch(const ExceptionConnection& ex)
+  {
+     connection_possible = false;
+     g_warning("debug Frame_Glom::create_database() exception caught: connection failed be");
+  }
+
+  if(!connection_possible)
+  {
+    g_warning("debug Frame_Glom::create_database(): connection was not possible.");
+    return false;
+  }
+  else
+  {
+    //This must now succeed, because we've already tried it once:
+    sharedptr<SharedConnection> sharedconnection;
+    try
+    {
+      if(request_password)
+        sharedconnection = m_pDialogConnection->connect_to_server_with_connection_settings();
+      else
+        sharedconnection = ConnectionPool::get_instance()->connect();
+    }
+    catch(const ExceptionConnection& ex)
+    {
+      g_warning("debug Frame_Glom::create_database() Connection failed.");
+ 
+      return false;
+    }
+      
+    if(sharedconnection)
+    {
+      Bakery::BusyCursor(*get_app_window());
+
+      Glib::RefPtr<Gnome::Gda::Connection> connection = sharedconnection->get_gda_connection();
+      if(connection)
+      {
+        bool result = connection->create_database(database_name);
+        if(result)
+        {
+          g_warning("Frame_Glom::create_database(): Succeeded: database_name=%s", database_name.c_str());
+          return true; // It succeeded.
+        }
+        else
+        {
+          //Tell the user:
+          Gtk::Dialog* dialog = 0;
+          try
+          {
+            Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_error_create_database");
+
+            refXml->get_widget("dialog_error_create_database", dialog);
+          }
+          catch(const Gnome::Glade::XmlError& ex)
+          {
+            std::cerr << ex.what() << std::endl;
+          }
+
+          dialog->run();
+          delete dialog;
+          return false; //Failed. //TODO: Allow the user to try with a different user name?
+        }
+      }
+    }
+  }
+ 
+  return false;
+}
+
+
+
  
 
 
