@@ -50,7 +50,7 @@ public:
   virtual void set_databases_selected(const Glib::ustring& strName);
   virtual void on_box_tables_selected(Glib::ustring strName);
 
-  virtual void on_menu_userlevel_Developer(const Glib::RefPtr<Gtk::RadioAction>& action);
+  virtual void on_menu_userlevel_Developer(const Glib::RefPtr<Gtk::RadioAction>& action, const Glib::RefPtr<Gtk::RadioAction>& operator_action);
   virtual void on_menu_userlevel_Operator(const Glib::RefPtr<Gtk::RadioAction>& action);
       
   virtual void on_menu_Mode_Data();
@@ -135,7 +135,6 @@ protected:
   Gtk::Dialog* m_pDialogConnectionFailed;
 
   Box_Data_List_Related m_HackToFixLinkerError; //The implementation of this class does not seem to be in the library unless I do this. murrayc.
-
 };
 
 #endif //FRAME_GLOM_H
