@@ -22,6 +22,7 @@
 #define BOX_DATA_LIST_H
 
 #include "box_data.h"
+#include "../utility_widgets/db_adddel/db_adddel_withbuttons.h"
 
 class Box_Data_List : public Box_Data
 {
@@ -33,6 +34,7 @@ public:
   virtual Gnome::Gda::Value get_primary_key_value_selected();
   
   virtual Gnome::Gda::Value get_entered_field_data(const Field& field) const;
+  virtual void set_entered_field_data(const Field& field, const Gnome::Gda::Value& value);
   
   virtual guint get_records_count() const;
     
@@ -51,6 +53,9 @@ protected:
   virtual void fill_from_database(); //override.
   virtual void fill_column_titles();
 
+  virtual bool get_field_primary_key_index(guint& field_column) const; //TODO: visible 
+  virtual bool get_field_primary_key(Field& field) const;
+
   void do_lookups(const Gtk::TreeModel::iterator& row, const Field& field_changed, const Gnome::Gda::Value& field_value, const Field& primary_key, const Gnome::Gda::Value& primary_key_value);
 
   //Signal handlers:
@@ -59,6 +64,8 @@ protected:
   virtual void on_adddel_user_requested_delete(const Gtk::TreeModel::iterator& rowStart, const Gtk::TreeModel::iterator& rowEnd);
   virtual void on_adddel_user_added(const Gtk::TreeModel::iterator& row);
   virtual void on_adddel_user_reordered_columns();
+  
+  virtual void on_adddel_user_requested_layout();
 
   virtual void on_adddel_user_changed(const Gtk::TreeModel::iterator& row, guint col);
 
@@ -67,10 +74,9 @@ protected:
   virtual bool get_field_column_index(const Glib::ustring& field_name, guint& index) const;
   
   //Member widgers:
-  mutable AddDel_WithButtons m_AddDel; //mutable because its get_ methods aren't const.
+  mutable DbAddDel_WithButtons m_AddDel; //mutable because its get_ methods aren't const.
 
   bool m_has_one_or_more_records;
-  guint m_first_col;
 
   type_signal_user_requested_details m_signal_user_requested_details;
 };
