@@ -28,8 +28,7 @@ Box_Tables::Box_Tables(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
   m_colTableName(0),
   m_colHidden(0),
   m_colTitle(0),
-  m_colDefault(0),
-  m_modified(false)
+  m_colDefault(0)
 {
   m_strHint = gettext("Select a table and click [Edit]. You may create a new table by entering the name in the last row.");
 
@@ -47,10 +46,10 @@ Box_Tables::Box_Tables(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade:
   refGlade->get_widget("checkbutton_show_hidden", m_pCheckButtonShowHidden);
   m_pCheckButtonShowHidden->signal_toggled().connect(sigc::mem_fun(*this, &Box_Tables::on_show_hidden_toggled));
   
-  m_AddDel.signal_user_added().connect(sigc::mem_fun(*this, &Box_Tables::on_AddDel_Add));
-  m_AddDel.signal_user_requested_delete().connect(sigc::mem_fun(*this, &Box_Tables::on_AddDel_Delete));
-  m_AddDel.signal_user_requested_edit().connect(sigc::mem_fun(*this, &Box_Tables::on_AddDel_Edit));
-  m_AddDel.signal_user_changed().connect(sigc::mem_fun(*this, &Box_Tables::on_AddDel_changed));
+  m_AddDel.signal_user_added().connect(sigc::mem_fun(*this, &Box_Tables::on_adddel_Add));
+  m_AddDel.signal_user_requested_delete().connect(sigc::mem_fun(*this, &Box_Tables::on_adddel_Delete));
+  m_AddDel.signal_user_requested_edit().connect(sigc::mem_fun(*this, &Box_Tables::on_adddel_Edit));
+  m_AddDel.signal_user_changed().connect(sigc::mem_fun(*this, &Box_Tables::on_adddel_changed));
 
   show_all_children();
 }
@@ -65,7 +64,7 @@ void Box_Tables::fill_table_row(const Gtk::TreeModel::iterator& iter, const Tabl
   {
     m_AddDel.set_value_key(iter, table_info.m_name);
     m_AddDel.set_value(iter, m_colTableName, table_info.m_name);
-    m_AddDel.set_value(iter, m_colHidden, table_info.m_name);
+    m_AddDel.set_value(iter, m_colHidden, table_info.m_hidden);
     m_AddDel.set_value(iter, m_colTitle, table_info.m_title);
     m_AddDel.set_value(iter, m_colDefault, table_info.m_default);
   }
@@ -79,8 +78,15 @@ void Box_Tables::fill_from_database()
     
   //Enable/Disable extra widgets:
   bool developer_mode = (get_userlevel() == AppState::USERLEVEL_DEVELOPER);
+
+  //Developers see more columns, so make it bigger:
+  if(developer_mode)
+    set_size_request(400, -1);
+  else
+    set_size_request(-1, -1);
+  
   m_pCheckButtonShowHidden->set_sensitive(developer_mode); //Operators have no choice - they can't see hidden tables ever.
-  if( get_userlevel() != AppState::USERLEVEL_DEVELOPER)
+  if(!developer_mode)
     m_pCheckButtonShowHidden->set_active(false); //Operators have no choice - they can't see hidden tables ever.
 
   m_AddDel.remove_all();
@@ -88,18 +94,17 @@ void Box_Tables::fill_from_database()
   //Add the columns:
   m_AddDel.remove_all_columns();
 
-  bool editable = developer_mode;
-  m_colTableName = m_AddDel.add_column(gettext("Tables"), AddDelColumnInfo::STYLE_Text, editable);
-
-  bool visible_extras = developer_mode;
-  m_colHidden = m_AddDel.add_column(gettext("Hidden"), AddDelColumnInfo::STYLE_Boolean, true, visible_extras);
-  m_colTitle =  m_AddDel.add_column(gettext("Title"), AddDelColumnInfo::STYLE_Text, true, visible_extras);
-  m_colDefault =  m_AddDel.add_column(gettext("Default"), AddDelColumnInfo::STYLE_Boolean,  true, visible_extras);
+  const bool editable = developer_mode;
+  const bool visible_extras = developer_mode;
+  m_colTableName = m_AddDel.add_column(gettext("Tables"), AddDelColumnInfo::STYLE_Text, editable, visible_extras);
+  m_colHidden = m_AddDel.add_column(gettext("Hidden"), AddDelColumnInfo::STYLE_Boolean, editable, visible_extras);
+  m_colTitle =  m_AddDel.add_column(gettext("Title"), AddDelColumnInfo::STYLE_Text, editable, true);
+  m_colDefault =  m_AddDel.add_column(gettext("Default"), AddDelColumnInfo::STYLE_Boolean,  editable, visible_extras);
   
 
   //gettext("Server: ") +  m_strServerName + ", " + 
-  Glib::ustring strTitle = Glib::ustring("<b>") + gettext("Tables from Database: ") + get_database_name() + "</b>";
-  m_pLabelFrameTitle->set_markup(strTitle);
+  //Glib::ustring strTitle = Glib::ustring("<b>") + gettext("Tables from Database: ") + get_database_name() + "</b>";
+  //m_pLabelFrameTitle->set_markup(strTitle);
 
   //Get the list of hidden tables:
 
@@ -164,12 +169,12 @@ void Box_Tables::fill_from_database()
 
   m_AddDel.set_allow_add(developer_mode);
   m_AddDel.set_allow_delete(developer_mode);
-      
-  m_modified = false;
 }
 
-void Box_Tables::on_AddDel_Add(const Gtk::TreeModel::iterator& row)
+void Box_Tables::on_adddel_Add(const Gtk::TreeModel::iterator& row)
 {
+  //TODO: Handle cell renderer changes to prevent illegal table names (e.g. starting with numbers.)"
+  
   Glib::ustring table_name = m_AddDel.get_value(row, m_colTableName);
   if(!table_name.empty())
   {
@@ -182,11 +187,12 @@ void Box_Tables::on_AddDel_Add(const Gtk::TreeModel::iterator& row)
 
     //PostgresSQL:
     //Query_execute( "CREATE TABLE " + table_name + " (" + primary_key_name + " serial NOT NULL  PRIMARY KEY)" );
-    Query_execute( "CREATE TABLE " + table_name + " (" + primary_key_name + " numeric NOT NULL  PRIMARY KEY)" );
+    Query_execute( "CREATE TABLE \"" + table_name + "\" (" + primary_key_name + " numeric NOT NULL  PRIMARY KEY)" );
 
     //Show the new information for this whole row:
     TableInfo table_info;
     table_info.m_name = table_name;
+    table_info.m_title = util_title_from_string( table_name ); //Start with a title that might be appropriate.
     fill_table_row(row, table_info);
     
     //Save the field information directly into the database, because we can not get all the correct information from the database.
@@ -208,12 +214,10 @@ void Box_Tables::on_AddDel_Add(const Gtk::TreeModel::iterator& row)
 
     save_to_document();
     //fill_from_database(); //We should not modify the model structure in a cellrenderer signal handler.
-
-    m_modified = true;
   }
 }
 
-void Box_Tables::on_AddDel_Delete(const Gtk::TreeModel::iterator& rowStart, const Gtk::TreeModel::iterator& rowEnd)
+void Box_Tables::on_adddel_Delete(const Gtk::TreeModel::iterator& rowStart, const Gtk::TreeModel::iterator& rowEnd)
 {
   Gtk::TreeModel::iterator iterAfter = rowEnd;
   ++iterAfter;
@@ -221,21 +225,35 @@ void Box_Tables::on_AddDel_Delete(const Gtk::TreeModel::iterator& rowStart, cons
   bool something_changed = false;
   for(Gtk::TreeModel::iterator iter = rowStart; iter != iterAfter; ++iter)
   {
-    Glib::ustring strName = m_AddDel.get_value_key(iter);
+    Glib::ustring table_name = m_AddDel.get_value_key(iter);
 
-    if(!strName.empty())
+    if(!table_name.empty())
     {
-      //Ask the user to confirm:
-      Glib::ustring strMsg = gettext("Are you sure that you want to delete this table?\nTable name: ")
-                          + strName;
-      Gtk::MessageDialog dialog(strMsg);
-      int iButtonClicked = dialog.run();
-
-      //Delete the table:
-      if(iButtonClicked == Gtk::RESPONSE_OK)
+      if(m_pDocument)
       {
-        Query_execute( "DROP TABLE " + strName);
-        something_changed = true;
+        //Don't open a table that the document does not know about, because we need information from the document:
+        if(!m_pDocument->get_table_is_known(table_name))
+        {
+           //TODO: Do not show tables that are not in the document.
+           Gtk::MessageDialog dialog(gettext("You can not delete this table, because there is no information about this table in the document."));
+           dialog.set_transient_for(*get_app_window());
+           dialog.run();
+        }
+        else
+        {
+          //Ask the user to confirm:
+          Glib::ustring strMsg = gettext("Are you sure that you want to delete this table?\nTable name: ")
+                              + table_name;
+          Gtk::MessageDialog dialog(strMsg);
+          int iButtonClicked = dialog.run();
+
+          //Delete the table:
+          if(iButtonClicked == Gtk::RESPONSE_OK)
+          {
+            Query_execute( "DROP TABLE " + table_name);
+            something_changed = true;
+          }
+        }
       }
     }
   }
@@ -245,12 +263,10 @@ void Box_Tables::on_AddDel_Delete(const Gtk::TreeModel::iterator& rowStart, cons
     save_to_document();
 
     fill_from_database();
-
-    m_modified = true;
   }
 }
 
-void Box_Tables::on_AddDel_Edit(const Gtk::TreeModel::iterator& row)
+void Box_Tables::on_adddel_Edit(const Gtk::TreeModel::iterator& row)
 {
   Glib::ustring table_name = m_AddDel.get_value_key(row);
   
@@ -280,26 +296,27 @@ void Box_Tables::save_to_document()
 {
   if(get_userlevel() == AppState::USERLEVEL_DEVELOPER)
   {
-    if(true); //m_modified
-    {
-      //Save the hidden tables. TODO_usermode: Only if we are in developer mode.
-      Document_Glom::type_listTableInfo listTables;
+    //Save the hidden tables. TODO_usermode: Only if we are in developer mode.
+    Document_Glom::type_listTableInfo listTables;
 
-      for(Gtk::TreeModel::iterator iter = m_AddDel.get_model()->children().begin(); iter != m_AddDel.get_model()->children().end(); ++iter)
+    for(Gtk::TreeModel::iterator iter = m_AddDel.get_model()->children().begin(); iter != m_AddDel.get_model()->children().end(); ++iter)
+    {
+      TableInfo table_info;
+      table_info.m_name = m_AddDel.get_value(iter, m_colTableName);
+
+      if(!table_info.m_name.empty())
       {
-        TableInfo table_info;
-        table_info.m_name = m_AddDel.get_value(iter, m_colTableName);
         table_info.m_hidden  = m_AddDel.get_value_as_bool(iter, m_colHidden);
         table_info.m_title  = m_AddDel.get_value(iter, m_colTitle);
         table_info.m_default  = m_AddDel.get_value_as_bool(iter, m_colDefault);
 
         listTables.push_back(table_info);
       }
+    }
 
-      if(m_pDocument)
-        m_pDocument->set_tables( listTables); //TODO: Don't save all new tables - just the ones already in the document.
-      }
-   }
+    if(m_pDocument)
+      m_pDocument->set_tables( listTables); //TODO: Don't save all new tables - just the ones already in the document.
+  }
 }
 
 void Box_Tables::on_show_hidden_toggled()
@@ -307,7 +324,7 @@ void Box_Tables::on_show_hidden_toggled()
   fill_from_database();
 }
 
-void Box_Tables::on_AddDel_changed(const Gtk::TreeModel::iterator& row, guint column)
+void Box_Tables::on_adddel_changed(const Gtk::TreeModel::iterator& row, guint column)
 {
   if(get_userlevel() == AppState::USERLEVEL_DEVELOPER)
   {
@@ -315,12 +332,10 @@ void Box_Tables::on_AddDel_changed(const Gtk::TreeModel::iterator& row, guint co
     {
       save_to_document();
       //TODO: This causes a crash. fill_from_database(); //Hide/show the table.
-      m_modified = true;
     }
     else if( (column == m_colTitle) || (column == m_colDefault) )
     {
       save_to_document();
-      m_modified = true;
     } 
     else if(column == m_colTableName)
     {
@@ -335,17 +350,28 @@ void Box_Tables::on_AddDel_changed(const Gtk::TreeModel::iterator& row, guint co
         //Rename the table:
         if(iButtonClicked == Gtk::RESPONSE_OK)
         {
-          Query_execute( "ALTER TABLE " + table_name + " RENAME TO " + table_name_new);
-          set_modified();
-          //fill_from_database(); //We should not modify the model structure in a cellrenderer signal handler.
+          bool test = Query_execute( "ALTER TABLE " + table_name + " RENAME TO " + table_name_new);
+          if(test)
+          {
+            //Change the AddDel item's key:
+            m_AddDel.set_value_key(row, table_name_new);
+            
+            set_modified();
+
+            //Tell the document that this table's name has changed:
+            Document_Glom* document = get_document();
+            if(document)
+              document->change_table_name(table_name, table_name_new);
+
+            //fill_from_database(); //We should not modify the model structure in a cellrenderer signal handler.
+          }
         }
       }
     }
   }
 }
 
-void  Box_Tables::on_userlevel_changed(AppState::userlevels /* userlevel */)
+void Box_Tables::on_userlevel_changed(AppState::userlevels /* userlevel */)
 {
   fill_from_database();
 }
-
